@@ -8,8 +8,8 @@ from abc import ABCMeta, abstractmethod, abstractproperty
 from enum import Enum
 from typing import Dict, List, Optional, Type, Union
 
-from ax.core.base import Base
 from ax.core.types import TParamValue
+from ax.utils.common.equality import Base
 
 
 FIXED_CHOICE_PARAM_ERROR = (
@@ -114,8 +114,8 @@ class RangeParameter(Parameter):
             name: Name of the parameter.
             parameter_type: Enum indicating the type of parameter
                 value (e.g. string, int).
-            lower: Lower bound of the parameter range.
-            upper: Upper bound of the parameter range.
+            lower: Lower bound of the parameter range (inclusive).
+            upper: Upper bound of the parameter range (inclusive).
             log_scale: Whether to sample in the log space when drawing
                 random values of the parameter.
             digits: Number of digits to round values to for float type.
@@ -153,8 +153,12 @@ class RangeParameter(Parameter):
             ParameterType.FLOAT,
         ):
             raise ValueError("RangeParameter type must be int or float.")
+        # pyre-fixme[58]: `>=` is not supported for operand types `Union[None, bool,
+        #  float, int, str]` and `Union[None, bool, float, int, str]`.
         if lower >= upper:
             raise ValueError("max must be strictly larger than min.")
+        # pyre-fixme[58]: `<=` is not supported for operand types `Union[None, bool,
+        #  float, int, str]` and `int`.
         if log_scale and lower <= 0:
             raise ValueError("Cannot take log when min <= 0.")
         if not (self.is_valid_type(lower)) or not (self.is_valid_type(upper)):
@@ -232,6 +236,8 @@ class RangeParameter(Parameter):
         # Re-scale min and max to new digits definition
         cast_lower = self.cast(self._lower)
         cast_upper = self.cast(self._upper)
+        # pyre-fixme[58]: `>=` is not supported for operand types `Union[None, bool,
+        #  float, int, str]` and `Union[None, bool, float, int, str]`.
         if cast_lower >= cast_upper:
             raise ValueError(
                 f"Lower bound {cast_lower} is >= upper bound {cast_upper}."
@@ -273,6 +279,8 @@ class RangeParameter(Parameter):
 
         # This might have issues with ints > 2^24
         if self.parameter_type is ParameterType.INT:
+            # pyre-fixme[6]: Expected `Union[_SupportsIndex, bytearray, bytes, str,
+            #  typing.SupportsFloat]` for 1st param but got `Union[None, float, str]`.
             return isinstance(value, int) or float(value).is_integer()
         return True
 
@@ -423,6 +431,7 @@ class ChoiceParameter(Parameter):
             name=self._name,
             parameter_type=self._parameter_type,
             values=self._values,
+            is_ordered=self._is_ordered,
             is_task=self._is_task,
             is_fidelity=self._is_fidelity,
             target_value=self._target_value,
