@@ -12,6 +12,7 @@ from ax.core.experiment import Experiment
 from ax.core.generator_run import GeneratorRun
 from ax.core.search_space import SearchSpace
 from ax.exceptions.core import DataRequiredError
+from ax.exceptions.generation_strategy import GenerationStrategyCompleted
 from ax.modelbridge.discrete import DiscreteModelBridge
 from ax.modelbridge.factory import get_sobol
 from ax.modelbridge.generation_strategy import (
@@ -25,6 +26,7 @@ from ax.modelbridge.torch import TorchModelBridge
 from ax.models.random.sobol import SobolGenerator
 from ax.utils.common.testutils import TestCase
 from ax.utils.testing.core_stubs import (
+    get_branin_data,
     get_branin_experiment,
     get_choice_parameter,
     get_data,
@@ -135,7 +137,7 @@ class TestGenerationStrategy(TestCase):
             steps=[GenerationStep(model=Models.SOBOL, num_trials=-1)]
         )
         self.assertEqual(
-            str(gs2), ("GenerationStrategy(name='Sobol', steps=[Sobol for all trials])")
+            str(gs2), "GenerationStrategy(name='Sobol', steps=[Sobol for all trials])"
         )
 
     def test_equality(self):
@@ -161,18 +163,30 @@ class TestGenerationStrategy(TestCase):
 
     def test_restore_from_generator_run(self):
         gs = GenerationStrategy(
-            steps=[
-                GenerationStep(model=Models.SOBOL, num_trials=5),
-                GenerationStep(model=Models.GPEI, num_trials=-1),
-            ]
+            steps=[GenerationStep(model=Models.SOBOL, num_trials=5)]
         )
+        # No generator runs on GS, so can't restore from one.
         with self.assertRaises(ValueError):
             gs._restore_model_from_generator_run()
-        gs.gen(experiment=get_branin_experiment())
+        exp = get_branin_experiment(with_batch=True)
+        gs.gen(experiment=exp)
         model = gs.model
-        gs._restore_model_from_generator_run()
-        # Model should be reset.
-        self.assertIsNot(model, gs.model)
+        # Create a copy of the generation strategy and check that when
+        # we restore from last generator run, the model will be set
+        # correctly and that `_seen_trial_indices_by_status` is filled.
+        new_gs = GenerationStrategy(
+            steps=[GenerationStep(model=Models.SOBOL, num_trials=5)]
+        )
+        new_gs._experiment = exp
+        new_gs._generator_runs = gs._generator_runs
+        self.assertIsNone(new_gs._seen_trial_indices_by_status)
+        new_gs._restore_model_from_generator_run()
+        self.assertEqual(gs._seen_trial_indices_by_status, exp.trial_indices_by_status)
+        # Model should be reset, but it should be the same model with same data.
+        self.assertIsNot(model, new_gs.model)
+        self.assertEqual(model.__class__, new_gs.model.__class__)  # Model bridge.
+        self.assertEqual(model.model.__class__, new_gs.model.model.__class__)  # Model.
+        self.assertEqual(model._training_data, new_gs.model._training_data)
 
     def test_min_observed(self):
         # We should fail to transition the next model if there is not
@@ -224,7 +238,7 @@ class TestGenerationStrategy(TestCase):
         self.assertEqual(sobol_GPEI.model_transitions, [5])
         # exp.new_trial(generator_run=sobol_GPEI.gen(exp)).run()
         for i in range(7):
-            g = sobol_GPEI.gen(exp, exp.fetch_data())
+            g = sobol_GPEI.gen(exp)
             exp.new_trial(generator_run=g).run()
             if i > 4:
                 self.mock_torch_model_bridge.assert_called()
@@ -252,9 +266,9 @@ class TestGenerationStrategy(TestCase):
                     },
                 )
                 self.assertEqual(g._model_state_after_gen, {"init_position": i + 1})
-        # Check completeness error message.
-        with self.assertRaisesRegex(ValueError, "Generation strategy"):
-            g = sobol_GPEI.gen(exp, exp.fetch_data())
+        # Check completeness error message when GS should be done.
+        with self.assertRaises(GenerationStrategyCompleted):
+            g = sobol_GPEI.gen(exp)
 
     def test_sobol_GPEI_strategy_keep_generating(self):
         exp = get_branin_experiment()
@@ -268,14 +282,15 @@ class TestGenerationStrategy(TestCase):
         self.assertEqual(sobol_GPEI_generation_strategy.model_transitions, [5])
         exp.new_trial(generator_run=sobol_GPEI_generation_strategy.gen(exp)).run()
         for i in range(1, 15):
-            g = sobol_GPEI_generation_strategy.gen(exp, exp.fetch_data())
+            g = sobol_GPEI_generation_strategy.gen(exp)
             exp.new_trial(generator_run=g).run()
             if i > 4:
                 self.assertIsInstance(
                     sobol_GPEI_generation_strategy.model, TorchModelBridge
                 )
 
-    def test_factorial_thompson_strategy(self):
+    @patch(f"{Experiment.__module__}.Experiment.fetch_data", return_value=get_data())
+    def test_factorial_thompson_strategy(self, _):
         exp = get_branin_experiment()
         factorial_thompson_generation_strategy = GenerationStrategy(
             steps=[
@@ -295,9 +310,7 @@ class TestGenerationStrategy(TestCase):
         self.assertEqual(kwargs.get("model_key"), "Factorial")
 
         # Subsequent Thompson sampling batch.
-        exp.new_batch_trial(
-            factorial_thompson_generation_strategy.gen(experiment=exp, data=get_data())
-        )
+        exp.new_batch_trial(factorial_thompson_generation_strategy.gen(experiment=exp))
         args, kwargs = mock_model_bridge._set_kwargs_to_save.call_args
         self.assertEqual(kwargs.get("model_key"), "Thompson")
 
@@ -321,7 +334,7 @@ class TestGenerationStrategy(TestCase):
             ]
         )
         exp = get_branin_experiment()
-        gs.gen(exp, exp.fetch_data())
+        gs.gen(exp)
         self.assertFalse(gs._model.model.scramble)
 
     def test_sobol_GPEI_strategy_batches(self):
@@ -347,15 +360,11 @@ class TestGenerationStrategy(TestCase):
         for i in range(1, 8):
             if i == 7:
                 # Check completeness error message.
-                with self.assertRaisesRegex(ValueError, "Generation strategy"):
-                    g = sobol_GPEI_generation_strategy.gen(exp, exp.fetch_data(), n=2)
+                with self.assertRaises(GenerationStrategyCompleted):
+                    g = sobol_GPEI_generation_strategy.gen(exp, n=2)
             else:
-                g = sobol_GPEI_generation_strategy.gen(
-                    exp, exp._fetch_trial_data(trial_index=i - 1), n=2
-                )
+                g = sobol_GPEI_generation_strategy.gen(exp, n=2)
             exp.new_batch_trial(generator_run=g).run()
-        with self.assertRaises(ValueError):
-            sobol_GPEI_generation_strategy.gen(exp, exp.fetch_data())
         self.assertIsInstance(sobol_GPEI_generation_strategy.model, TorchModelBridge)
 
     def test_with_factory_function(self):
@@ -418,3 +427,43 @@ class TestGenerationStrategy(TestCase):
         ).mark_running(no_runner_required=True)
         with self.assertRaises(MaxParallelismReachedException):
             sobol_generation_strategy.gen(experiment=exp)
+
+    @patch(f"{RandomModelBridge.__module__}.RandomModelBridge.update")
+    @patch(f"{Experiment.__module__}.Experiment.fetch_trials_data")
+    def test_use_update(self, mock_fetch_trials_data, mock_update):
+        exp = get_branin_experiment()
+        sobol_gs_with_update = GenerationStrategy(
+            steps=[GenerationStep(model=Models.SOBOL, num_trials=-1, use_update=True)]
+        )
+        # Try without passing data (generation strategy fetches data from experiment).
+        trial = exp.new_trial(generator_run=sobol_gs_with_update.gen(experiment=exp))
+        mock_update.assert_not_called()
+        trial._status = TrialStatus.COMPLETED
+        for i in range(3):
+            trial = exp.new_trial(
+                generator_run=sobol_gs_with_update.gen(experiment=exp)
+            )
+            self.assertEqual(
+                mock_fetch_trials_data.call_args[1].get("trial_indices"), {i}
+            )
+            trial._status = TrialStatus.COMPLETED
+        # `_seen_trial_indices_by_status` is set during `gen`, to the experiment's
+        # `trial_indices_by_Status` at the time of candidate generation.
+        self.assertNotEqual(
+            sobol_gs_with_update._seen_trial_indices_by_status,
+            exp.trial_indices_by_status,
+        )
+        # Try with passing data.
+        sobol_gs_with_update.gen(
+            experiment=exp, data=get_branin_data(trial_indices=range(4))
+        )
+        # Now `_seen_trial_indices_by_status` should be set to experiment's,
+        self.assertEqual(
+            sobol_gs_with_update._seen_trial_indices_by_status,
+            exp.trial_indices_by_status,
+        )
+        # Only the data for the last completed trial should be considered new and passed
+        # to `update`.
+        self.assertEqual(
+            set(mock_update.call_args[1].get("new_data").df["trial_index"].values), {3}
+        )
