@@ -16,6 +16,7 @@ from ax.core.data import Data
 from ax.core.experiment import Experiment
 from ax.core.generator_run import GeneratorRun
 from ax.core.metric import Metric
+from ax.core.multi_type_experiment import MultiTypeExperiment
 from ax.core.objective import MultiObjective, Objective, ScalarizedObjective
 from ax.core.optimization_config import OptimizationConfig
 from ax.core.outcome_constraint import OutcomeConstraint
@@ -34,27 +35,35 @@ from ax.core.search_space import SearchSpace
 from ax.core.simple_experiment import SimpleExperiment
 from ax.core.trial import Trial
 from ax.core.types import ComparisonOp
-from ax.metrics.branin import BraninMetric, NegativeBraninMetric
+from ax.metrics.branin import AugmentedBraninMetric, BraninMetric, NegativeBraninMetric
 from ax.metrics.factorial import FactorialMetric
-from ax.metrics.hartmann6 import Hartmann6Metric
+from ax.metrics.hartmann6 import AugmentedHartmann6Metric, Hartmann6Metric
 from ax.metrics.l2norm import L2NormMetric
 from ax.metrics.noisy_function import NoisyFunctionMetric
 from ax.modelbridge.factory import Models
 from ax.modelbridge.generation_strategy import GenerationStep, GenerationStrategy
 from ax.modelbridge.transforms.base import Transform
+from ax.models.torch.botorch_modular.acquisition import Acquisition
+from ax.models.torch.botorch_modular.model import BoTorchModel
+from ax.models.torch.botorch_modular.surrogate import Surrogate
 from ax.runners.synthetic import SyntheticRunner
+from ax.storage.json_store.decoders import class_from_json, transform_type_from_json
 from ax.storage.json_store.encoders import (
     arm_to_dict,
     batch_to_dict,
     benchmark_problem_to_dict,
+    botorch_model_to_dict,
+    botorch_modular_to_dict,
     choice_parameter_to_dict,
     data_to_dict,
     experiment_to_dict,
     fixed_parameter_to_dict,
+    generation_step_to_dict,
     generation_strategy_to_dict,
     generator_run_to_dict,
     metric_to_dict,
     multi_objective_to_dict,
+    multi_type_experiment_to_dict,
     objective_to_dict,
     observation_features_to_dict,
     optimization_config_to_dict,
@@ -67,28 +76,37 @@ from ax.storage.json_store.encoders import (
     search_space_to_dict,
     simple_experiment_to_dict,
     sum_parameter_constraint_to_dict,
+    surrogate_to_dict,
     transform_type_to_dict,
     trial_to_dict,
 )
 from ax.storage.utils import DomainType, ParameterConstraintType
+from botorch.acquisition.acquisition import AcquisitionFunction
+from botorch.models.model import Model
+from gpytorch.mlls.marginal_log_likelihood import MarginalLogLikelihood
 
 
 ENCODER_REGISTRY: Dict[Type, Callable[[Any], Dict[str, Any]]] = {
     Arm: arm_to_dict,
+    AugmentedBraninMetric: metric_to_dict,
+    AugmentedHartmann6Metric: metric_to_dict,
     BatchTrial: batch_to_dict,
     BenchmarkProblem: benchmark_problem_to_dict,
+    BoTorchModel: botorch_model_to_dict,
     BraninMetric: metric_to_dict,
     ChoiceParameter: choice_parameter_to_dict,
     Data: data_to_dict,
     Experiment: experiment_to_dict,
     FactorialMetric: metric_to_dict,
     FixedParameter: fixed_parameter_to_dict,
+    GenerationStep: generation_step_to_dict,
     GenerationStrategy: generation_strategy_to_dict,
     GeneratorRun: generator_run_to_dict,
     Hartmann6Metric: metric_to_dict,
     L2NormMetric: metric_to_dict,
     Metric: metric_to_dict,
     MultiObjective: multi_objective_to_dict,
+    MultiTypeExperiment: multi_type_experiment_to_dict,
     NegativeBraninMetric: metric_to_dict,
     NoisyFunctionMetric: metric_to_dict,
     Objective: objective_to_dict,
@@ -102,18 +120,33 @@ ENCODER_REGISTRY: Dict[Type, Callable[[Any], Dict[str, Any]]] = {
     SimpleBenchmarkProblem: benchmark_problem_to_dict,
     SimpleExperiment: simple_experiment_to_dict,
     SumConstraint: sum_parameter_constraint_to_dict,
+    Surrogate: surrogate_to_dict,
     SyntheticRunner: runner_to_dict,
     Trial: trial_to_dict,
-    Type[Transform]: transform_type_to_dict,
     ObservationFeatures: observation_features_to_dict,
 }
 
+# Registry for class types, not instances.
+# NOTE: Avoid putting a class along with its subclass in `CLASS_ENCODER_REGISTRY`.
+# The encoder iterates through this dictionary and uses the first superclass that
+# it finds, which might not be the intended superclass.
+CLASS_ENCODER_REGISTRY: Dict[Type, Callable[[Any], Dict[str, Any]]] = {
+    Acquisition: botorch_modular_to_dict,
+    AcquisitionFunction: botorch_modular_to_dict,
+    MarginalLogLikelihood: botorch_modular_to_dict,
+    Model: botorch_modular_to_dict,
+    Transform: transform_type_to_dict,
+}
+
 DECODER_REGISTRY: Dict[str, Type] = {
     "AbandonedArm": AbandonedArm,
+    "AugmentedBraninMetric": AugmentedBraninMetric,
+    "AugmentedHartmann6Metric": AugmentedHartmann6Metric,
     "Arm": Arm,
     "BatchTrial": BatchTrial,
     "BenchmarkProblem": BenchmarkProblem,
     "BenchmarkResult": BenchmarkResult,
+    "BoTorchModel": BoTorchModel,
     "BraninMetric": BraninMetric,
     "ChoiceParameter": ChoiceParameter,
     "ComparisonOp": ComparisonOp,
@@ -131,6 +164,7 @@ DECODER_REGISTRY: Dict[str, Type] = {
     "Metric": Metric,
     "Models": Models,
     "MultiObjective": MultiObjective,
+    "MultiTypeExperiment": MultiTypeExperiment,
     "NegativeBraninMetric": NegativeBraninMetric,
     "NoisyFunctionMetric": NoisyFunctionMetric,
     "Objective": Objective,
@@ -146,9 +180,19 @@ DECODER_REGISTRY: Dict[str, Type] = {
     "SimpleBenchmarkProblem": SimpleBenchmarkProblem,
     "SimpleExperiment": SimpleExperiment,
     "SumConstraint": SumConstraint,
+    "Surrogate": Surrogate,
     "SyntheticRunner": SyntheticRunner,
     "Trial": Trial,
     "TrialStatus": TrialStatus,
-    "Type[Transform]": Type[Transform],
     "ObservationFeatures": ObservationFeatures,
 }
+
+
+# Registry for class types, not instances.
+CLASS_DECODER_REGISTRY: Dict[str, Callable[[Dict[str, Any]], Any]] = {
+    "Type[Acquisition]": class_from_json,
+    "Type[AcquisitionFunction]": class_from_json,
+    "Type[MarginalLogLikelihood]": class_from_json,
+    "Type[Model]": class_from_json,
+    "Type[Transform]": transform_type_from_json,
+}
