@@ -4,10 +4,10 @@
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
-from contextlib import contextmanager
+from contextlib import contextmanager, nullcontext
 from datetime import datetime
 from enum import Enum
-from typing import Any, Callable, Generator, List, Optional, TypeVar
+from typing import Any, Callable, ContextManager, Generator, List, Optional, TypeVar
 
 import numpy as np
 from ax.exceptions.storage import ImmutabilityError
@@ -35,7 +35,6 @@ MEDIUMTEXT_BYTES: int = 2 ** 24 - 1
 LONGTEXT_BYTES: int = 2 ** 32 - 1
 
 # global database variables
-Ax_PROD_TIER: str = "xdb.adaptive_experiment"
 SESSION_FACTORY: Optional[Session] = None
 
 # set this to false to prevent SQLAlchemy for automatically expiring objects
@@ -82,10 +81,6 @@ class SQABase:
                 if isinstance(x, SQABase):
                     equal = x.equals(y)
                 elif isinstance(x, (int, float, str, bool, dict, Enum)):
-                    # pyre-fixme[29]: `Union[Callable[[object], bool],
-                    #  Callable[[object], bool], Callable[[object], bool],
-                    #  Callable[[object], bool], Callable[[object], bool],
-                    #  Callable[[object], bool]]` is not a function.
                     equal = x == y
                 else:
                     raise ValueError(
@@ -456,3 +451,11 @@ def session_scope() -> Generator[Session, None, None]:
         raise  # pragma: no cover
     finally:
         session.close()
+
+
+def optional_session_scope(
+    session: Optional[Session] = None,
+) -> ContextManager[Session]:
+    if session is not None:
+        return nullcontext(session)
+    return session_scope()
