@@ -6,7 +6,9 @@
 
 from datetime import datetime
 
+import pandas as pd
 from ax.utils.common.equality import (
+    dataframe_equals,
     datetime_equals,
     equality_typechecker,
     same_elements,
@@ -33,3 +35,12 @@ class EqualityTest(TestCase):
         self.assertTrue(datetime_equals(None, None))
         self.assertFalse(datetime_equals(None, now))
         self.assertTrue(datetime_equals(now, now))
+
+    def testDataframeEquals(self):
+        pd1 = pd.DataFrame.from_records([{"x": 100, "y": 200}])
+        pd2 = pd.DataFrame.from_records([{"y": 200, "x": 100}])
+        pd3 = pd.DataFrame.from_records([{"x": 100, "y": 300}])
+
+        self.assertTrue(dataframe_equals(pd.DataFrame(), pd.DataFrame()))
+        self.assertTrue(dataframe_equals(pd1, pd2))
+        self.assertFalse(dataframe_equals(pd1, pd3))
